## -*- coding: utf-8 -*-
<%inherit file="/base/base.html"/>

<%block name="title">
    ${_('Add user group')}
</%block>
<%def name="breadcrumbs_links()">
    ${h.link_to(_('Admin'),h.url('admin_home'))}
    &raquo;
    ${h.link_to(_('User Groups'),h.url('users_groups'))}
    &raquo;
    ${_('Add User Group')}
</%def>

<%block name="header_menu">
    ${self.menu('admin')}
</%block>

<%def name="main()">
<div class="box">
    <!-- box / title -->
    <div class="title">
        ${self.breadcrumbs()}
    </div>
    <!-- end box / title -->
    ${h.form(url('users_groups'))}
    <div class="form">
        <!-- fields -->
        <div class="fields">
             <div class="field">
                <div class="label">
                    <label for="users_group_name">${_('Group name')}:</label>
                </div>
                <div class="input">
                    ${h.text('users_group_name',class_='small')}
                </div>
             </div>
            <div class="field">
                <div class="label label-textarea">
                    <label for="user_group_description">${_('Description')}:</label>
                </div>
                <div class="textarea text-area editor">
                    ${h.textarea('user_group_description')}
                    <span class="help-block">${_('Short, optional description for this user group.')}</span>
                </div>
             </div>
             <div class="field">
                <div class="label label-checkbox">
                    <label for="users_group_active">${_('Active')}:</label>
                </div>
                <div class="checkboxes">
                    ${h.checkbox('users_group_active',value=True, checked='checked')}
                </div>
             </div>

            <div class="buttons">
              ${h.submit('save',_('Save'),class_="btn")}
            </div>
        </div>
    </div>
    ${h.end_form()}
</div>

<script>
    $(document).ready(function(){
        $('#users_group_name').focus();
    });
</script>
</%def>
