#!/usr/bin/env python2
# -*- coding: utf-8 -*-

from kallithea.lib.dbmigrate.migrate.versioning.script.base import BaseScript
from kallithea.lib.dbmigrate.migrate.versioning.script.py import PythonScript
from kallithea.lib.dbmigrate.migrate.versioning.script.sql import SqlScript
