## -*- coding: utf-8 -*-
<%inherit file="base/root.html"/>

<%block name="title">
    ${_('Password Reset')}
</%block>

<div id="register">
    <%include file="/base/flash_msg.html"/>
    <div class="title withlogo">
        %if c.site_name:
            <h5>${_('Reset Your Password to %s') % c.site_name}</h5>
        %else:
            <h5>${_('Reset Your Password')}</h5>
        %endif
    </div>
    <div class="inner">
        ${h.form(url('password_reset'))}
        <div class="form">
            <!-- fields -->
            <div class="fields">

                 <div class="field">
                    <div class="label">
                        <label for="email">${_('Email Address')}:</label>
                    </div>
                    <div class="input">
                        ${h.text('email')}
                    </div>
                 </div>

                %if c.captcha_active:
                <div class="field">
                    <div class="label">
                        <label for="email">${_('Captcha')}:</label>
                    </div>
                    <div class="input">
                        ${h.hidden('recaptcha_field')}
                        <div id="recaptcha"></div>
                    </div>
                </div>
                %endif

                <div class="buttons">
                    <div class="nohighlight">
                      ${h.submit('send',_('Send Password Reset Email'),class_="btn")}
                          <div class="activation_msg">${_('A password reset link will be sent to the specified email address if it is registered in the system.')}</div>
                    </div>
                </div>
            </div>
        </div>
        ${h.end_form()}
        %if c.captcha_active:
        <script type="text/javascript" src="https://www.google.com/recaptcha/api/js/recaptcha_ajax.js"></script>
        %endif
        <script type="text/javascript">
         $(document).ready(function(){
            $('#email').focus();
            %if c.captcha_active:
            Recaptcha.create("${c.captcha_public_key}", "recaptcha",
                {
                  theme: "white"
                }
            );
            %endif
         });
        </script>
    </div>
   </div>
