## -*- coding: utf-8 -*-
<%inherit file="main.html"/>

%if is_mention:
<p>${_('%s mentioned you on %s pull request "%s"') % (pr_user_created, repo_name, pr_title)}</p>
%else:
<p>${_('%s requested your review of %s pull request "%s"') % (pr_user_created, repo_name, pr_title)}</p>
%endif

<p>${_('URL')}: <a href="${pr_url}">${pr_url}</a></p>

<p>${_('Description')}:</p>
<p style="white-space: pre-wrap;">${body}</p>

<p>${_('Changesets')}:</p>
<p style="white-space: pre-wrap;">
%for r,r_msg in pr_revisions:
<i>${h.short_id(r)}</i>:
${h.shorter(r_msg, 256)}

%endfor
</p>
