## -*- coding: utf-8 -*-
${h.form(url('repos_group',group_name=c.repo_group.group_name),method='put')}
<div class="form">
    <!-- fields -->
    <div class="fields">
        <div class="field">
            <div class="label">
                <label for="group_name">${_('Group name')}:</label>
            </div>
            <div class="input">
                ${h.text('group_name',class_='medium')}
            </div>
        </div>

        <div class="field">
            <div class="label label-textarea">
                <label for="group_description">${_('Description')}:</label>
            </div>
            <div class="textarea text-area editor">
                ${h.textarea('group_description',cols=23,rows=5,class_="medium")}
            </div>
        </div>

        <div class="field">
            <div class="label">
                <label for="group_parent_id">${_('Group parent')}:</label>
            </div>
            <div class="input">
                ${h.select('group_parent_id','',c.repo_groups,class_="medium")}
            </div>
        </div>
        <div class="field">
            <div class="label label-checkbox">
                <label for="enable_locking">${_('Enable locking')}:</label>
            </div>
            <div class="checkboxes">
                ${h.checkbox('enable_locking',value="True")}
                <span class="help-block">${_('Enable lock-by-pulling on group. This option will be applied to all other groups and repositories inside')}</span>
            </div>
        </div>
        <div class="buttons">
          ${h.submit('save',_('Save'),class_="btn")}
          ${h.reset('reset',_('Reset'),class_="btn")}
        </div>
    </div>
</div>
${h.end_form()}

${h.form(url('delete_repo_group', group_name=c.repo_group.group_name),method='delete')}
<div class="form">
    <div class="fields">
        <div class="field" style="border:none;color:#888">
            ${h.submit('remove_%s' % c.repo_group.group_name,_('Remove this group'),class_="btn red",onclick="return confirm('"+_('Confirm to delete this group')+"');")}
        </div>
    </div>
</div>
${h.end_form()}

<script>
    $(document).ready(function(){
        $("#group_parent_id").select2({
            'dropdownAutoWidth': true
        });
    });
</script>
