## -*- coding: utf-8 -*-
<%inherit file="/base/base.html"/>

<%block name="title">
    ${_('Add Repository')}
</%block>

<%def name="breadcrumbs_links()">
    %if c.authuser.is_admin:
    ${h.link_to(_('Admin'),h.url('admin_home'))}
    &raquo;
    ${h.link_to(_('Repositories'),h.url('repos'))}
    %else:
    ${_('Admin')}
    &raquo;
    ${_('Repositories')}
    %endif
    &raquo;
    ${_('Add Repository')}
</%def>

<%block name="header_menu">
    ${self.menu('admin')}
</%block>

<%def name="main()">
    <div class="box">
        <!-- box / title -->
        <div class="title">
            ${self.breadcrumbs()}
        </div>
        <%include file="repo_add_base.html"/>
    </div>
</%def>
