## -*- coding: utf-8 -*-
<%inherit file="/base/base.html"/>
<%block name="title">
    ${_('About')}
</%block>
<%def name="breadcrumbs()">
    ${c.site_name}
</%def>
<%block name="header_menu">
    ${self.menu('about')}
</%block>
<%def name="main()">

<div class="box">
  <!-- box / title -->
  <div class="title">
    <h5>${_('About')} Kallithea</h5>
  </div>

  <p><a href="https://kallithea-scm.org/">Kallithea</a> is a project of the
  <a href="http://sfconservancy.org/">Software Freedom Conservancy, Inc.</a>
  and is released under the terms of the
  <a href="http://www.gnu.org/copyleft/gpl.html">GNU General Public License,
  v 3.0 (GPLv3)</a>.</p>

  <p>Kallithea is copyrighted by various authors, including but not
  necessarily limited to the following:
  <ul style="margin: 0 0 0 50px;">

  <li>Copyright &copy; 2012&ndash;2016, Mads Kiilerich</li>
  <li>Copyright &copy; 2012&ndash;2016, Takumi IINO</li>
  <li>Copyright &copy; 2012&ndash;2016, Unity Technologies</li>
  <li>Copyright &copy; 2016, Robert James Dennington</li>
  <li>Copyright &copy; 2012, 2014&ndash;2015, Andrew Shadura</li>
  <li>Copyright &copy; 2012&ndash;2013, 2015, Aras Pranckevičius</li>
  <li>Copyright &copy; 2014&ndash;2015, Christian Oyarzun</li>
  <li>Copyright &copy; 2014&ndash;2015, Joseph Rivera</li>
  <li>Copyright &copy; 2014&ndash;2015, Michal Čihař</li>
  <li>Copyright &copy; 2014&ndash;2015, Sean Farley</li>
  <li>Copyright &copy; 2014&ndash;2015, Thomas De Schampheleire</li>
  <li>Copyright &copy; 2015, Anatoly Bubenkov</li>
  <li>Copyright &copy; 2015, Andrew Bartlett</li>
  <li>Copyright &copy; 2015, Balázs Úr</li>
  <li>Copyright &copy; 2015, Ben Finney</li>
  <li>Copyright &copy; 2015, Branko Majic</li>
  <li>Copyright &copy; 2015, Daniel Hobley</li>
  <li>Copyright &copy; 2015, David Avigni</li>
  <li>Copyright &copy; 2015, Denis Blanchette</li>
  <li>Copyright &copy; 2015, duanhongyi</li>
  <li>Copyright &copy; 2015, EriCSN Chang</li>
  <li>Copyright &copy; 2015, Étienne Gilli</li>
  <li>Copyright &copy; 2015, Grzegorz Krason</li>
  <li>Copyright &copy; 2015, Jan Heylen</li>
  <li>Copyright &copy; 2015, Kazunari Kobayashi</li>
  <li>Copyright &copy; 2015, Kevin Bullock</li>
  <li>Copyright &copy; 2015, kobanari</li>
  <li>Copyright &copy; 2015, Marc Abramowitz</li>
  <li>Copyright &copy; 2015, Marc Villetard</li>
  <li>Copyright &copy; 2015, Matthias Zilk</li>
  <li>Copyright &copy; 2015, Michael Pohl</li>
  <li>Copyright &copy; 2015, Michael V. DePalatis</li>
  <li>Copyright &copy; 2015, Morten Skaaning</li>
  <li>Copyright &copy; 2015, Nick High</li>
  <li>Copyright &copy; 2015, Niemand Jedermann</li>
  <li>Copyright &copy; 2015, Peter Vitt</li>
  <li>Copyright &copy; 2015, Robert Martinez</li>
  <li>Copyright &copy; 2015, Robert Rauch</li>
  <li>Copyright &copy; 2015, Ronny Pfannschmidt</li>
  <li>Copyright &copy; 2015, Sam Jaques</li>
  <li>Copyright &copy; 2015, Søren Løvborg</li>
  <li>Copyright &copy; 2015, Tuux</li>
  <li>Copyright &copy; 2015, Viktar Palstsiuk</li>
  <li>Copyright &copy; 2012, 2014, Dominik Ruf</li>
  <li>Copyright &copy; 2014, Bradley M. Kuhn</li>
  <li>Copyright &copy; 2014, Calinou</li>
  <li>Copyright &copy; 2014, Daniel Anderson</li>
  <li>Copyright &copy; 2014, Henrik Stuart</li>
  <li>Copyright &copy; 2014, Ingo von Borstel</li>
  <li>Copyright &copy; 2014, Jelmer Vernooĳ</li>
  <li>Copyright &copy; 2014, Jim Hague</li>
  <li>Copyright &copy; 2014, Matt Fellows</li>
  <li>Copyright &copy; 2014, Max Roman</li>
  <li>Copyright &copy; 2014, Na'Tosha Bard</li>
  <li>Copyright &copy; 2014, Rasmus Selsmark</li>
  <li>Copyright &copy; 2014, Tim Freund</li>
  <li>Copyright &copy; 2014, Travis Burtrum</li>
  <li>Copyright &copy; 2014, Zoltan Gyarmati</li>
  <li>Copyright &copy; 2010&ndash;2013, Marcin Kuźmiński</li>
  <li>Copyright &copy; 2010&ndash;2013, RhodeCode GmbH</li>
  <li>Copyright &copy; 2011, 2013, Aparkar</li>
  <li>Copyright &copy; 2012&ndash;2013, xpol</li>
  <li>Copyright &copy; 2013, Dennis Brakhane</li>
  <li>Copyright &copy; 2013, Grzegorz Rożniecki</li>
  <li>Copyright &copy; 2013, Jonathan Sternberg</li>
  <li>Copyright &copy; 2013, Leonardo Carneiro</li>
  <li>Copyright &copy; 2013, Magnus Ericmats</li>
  <li>Copyright &copy; 2013, Martin Vium</li>
  <li>Copyright &copy; 2013, Simon Lopez</li>
  <li>Copyright &copy; 2011&ndash;2012, Augusto Herrmann</li>
  <li>Copyright &copy; 2012, Dan Sheridan</li>
  <li>Copyright &copy; 2012, H Waldo G</li>
  <li>Copyright &copy; 2012, hppj</li>
  <li>Copyright &copy; 2012, Indra Talip</li>
  <li>Copyright &copy; 2012, mikespook</li>
  <li>Copyright &copy; 2012, nansenat16</li>
  <li>Copyright &copy; 2012, Philip Jameson</li>
  <li>Copyright &copy; 2012, Raoul Thill</li>
  <li>Copyright &copy; 2012, Tony Bussieres</li>
  <li>Copyright &copy; 2012, Vincent Duvert</li>
  <li>Copyright &copy; 2012, Vladislav Poluhin</li>
  <li>Copyright &copy; 2012, Zachary Auclair</li>
  <li>Copyright &copy; 2011, Ankit Solanki</li>
  <li>Copyright &copy; 2011, Dmitri Kuznetsov</li>
  <li>Copyright &copy; 2011, Jared Bunting</li>
  <li>Copyright &copy; 2011, Jason Harris</li>
  <li>Copyright &copy; 2011, Les Peabody</li>
  <li>Copyright &copy; 2011, Liad Shani</li>
  <li>Copyright &copy; 2011, Lorenzo M. Catucci</li>
  <li>Copyright &copy; 2011, Matt Zuba</li>
  <li>Copyright &copy; 2011, Nicolas VINOT</li>
  <li>Copyright &copy; 2011, Shawn K. O'Shea</li>
  <li>Copyright &copy; 2010, Łukasz Balcerzak</li>

## We did not list the following copyright holders, given that they appeared
## to use for-profit company affiliations in their contribution in the
## Mercurial log and therefore I didn't know if copyright was theirs or
## their company's.
## Copyright &copy; 2011 Thayne Harbaugh <thayne@fusionio.com>
## Copyright &copy; 2012 Dies Koper <diesk@fast.au.fujitsu.com>
## Copyright &copy; 2012 Erwin Kroon <e.kroon@smartmetersolutions.nl>
## Copyright &copy; 2012 Vincent Caron <vcaron@bearstech.com>
##
## These contributors' contributions may not be copyrightable:
## philip.j@hostdime.com in 2012
## Stefan Engel <mail@engel-stefan.de> in 2012
## Ton Plomp <tcplomp@gmail.com> in 2013
##
</ul></p>

<p>The above are the copyright holders who have submitted direct
  contributions to the Kallithea repository.  In
  the <a href="https://kallithea-scm.org/repos/kallithea">Kallithea source
  code</a>, there is
  a <a href="https://kallithea-scm.org/repos/kallithea/files/tip/LICENSE.md">list
  of third-party libraries and code that Kallithea incorporates</a>.</p>

</div>

</%def>
