<dl class="dl-horizontal">
<%
 elems = [
    (_('Kallithea version'), h.literal('%s <b><span style="color:#036185; text-decoration: underline;cursor: pointer" id="check_for_update" >%s</span></b>' % (c.kallithea_version, _('Check for updates'))), ''),
    (_('Kallithea configuration file'), c.ini['__file__'], ''),
    (_('Python version'), c.py_version, ''),
    (_('Platform'), c.platform, ''),
    (_('Git version'), c.git_version, ''),
    (_('Git path'), c.ini.get('git_path'), ''),
    (_('Upgrade info endpoint'), h.literal('%s <br/><span style="color:#999999">%s.</span>' % (c.update_url, _('Note: please make sure this server can access this URL'))), ''),
 ]
%>

<div id="update_notice" style="display: none">
    <div style="padding: 5px 0px 5px 0px; color: #000000; font-weight: bold">${_('Checking for updates...')}</div>
</div>
%for dt, dd, tt in elems:
  <dt style="width:150px; text-align: left">${dt}:</dt>
  <dd style="margin-left: 160px" title="${tt}">${dd}</dd>
%endfor
</dl>

<h4>${_('Python Packages')}</h4>
<table class="table" style="margin:0px 0px 0px 0px">
  <colgroup>
      <col style="width:180px">
  </colgroup>
  <tbody>
      %for key, value in c.modules:
          <tr>
              <td style="padding-right:5px;">${key}</td>
              <td>${value}</td>
          </tr>
      %endfor
  </tbody>
</table>

<script>
    $('#check_for_update').click(function(e){
        var $update_notice = $('#update_notice');
        $update_notice.show();
        asynchtml("${h.url('admin_settings_system_update')}", $update_notice);
    });
</script>
