## -*- coding: utf-8 -*-
## upgrade block rendered afte on-click check

<div class="alert ${'alert-warning' if c.should_upgrade else 'alert-success'}">
<p style="padding: 2px 0px 5px 0px; margin: 0px">

%if c.should_upgrade:
    A <b>new version</b> is available:
    %if c.latest_data.get('title'):
        <b>${h.literal(c.latest_data['title'])}</b>
    %else:
        <b>${c.latest_ver}</b>
    %endif
%else:
    You already have the <b>latest</b> stable version.
%endif
</p>

% if c.should_upgrade and c.important_notices:
<div style="color: #5f5f5f; padding: 3px 0px 5px 0px;">Important notes for this release:</div>
    <ul>
    % for notice in c.important_notices:
        <li>- ${notice}</li>
    % endfor
    </ul>
% endif
</div>
