div.codeblock {
    overflow: auto;
    padding: 0px;
    border: 1px solid #ccc;
    background: #f8f8f8;
    font-size: 100%;
    line-height: 100%;
    /* new */
    line-height: 125%;
    -webkit-border-radius: 4px;
    -moz-border-radius: 4px;
    border-radius: 4px;
}
div.codeblock .code-header {
    border-bottom: 1px solid #CCCCCC;
    background: #EEEEEE;
    padding: 10px 0 5px 0;
}

div.codeblock .code-header .stats {
    clear: both;
    padding: 2px 8px 2px 14px;
    border-bottom: 1px solid rgb(204, 204, 204);
    height: 23px;
    margin-bottom: 6px;
}

div.codeblock .code-header .stats .left {
    float: left;
}
div.codeblock .code-header .stats .left.img {
    margin-top: -2px;
}
div.codeblock .code-header .stats .left.item {
    float: left;
    padding: 0 9px 0 9px;
    border-right: 1px solid #ccc;
}
div.codeblock .code-header .stats .left.item pre {
}
div.codeblock .code-header .stats .left.item.last {
    border-right: none;
}
div.codeblock .code-header .stats .buttons {
    float: right;
    padding-right: 4px;
}

div.codeblock .code-header .author {
    margin-left: 15px;
    font-weight: bold;
    height: 25px;
}
div.codeblock .code-header .author .user {
    padding-top: 3px;
}
div.codeblock .code-header .commit {
    margin-left: 15px;
    font-weight: normal;
    white-space: pre;
}

.code-highlighttable,
div.codeblock .code-body table {
    width: 0 !important;
    border: 0px !important;
}

.code-highlighttable,
div.codeblock .code-body table td {
    border: 0px !important;
}

div.code-body {
    background-color: #FFFFFF;
}

div.codeblock .code-header .search-path {
    padding: 0px 0px 0px 10px;
}

div.search-code-body {
    background-color: #FFFFFF;
    padding: 5px 0px 5px 10px;
}

div.search-code-body pre .match {
    background-color: #FAFFA6;
}
div.search-code-body pre .break {
    background-color: #DDE7EF;
    width: 100%;
    color: #747474;
    display: block;
}
div.annotatediv {
    margin-left: 2px;
    margin-right: 4px;
}
.code-highlight {
    padding: 0px;
    margin-top: 5px;
    margin-bottom: 5px;
    border-left: 1px solid #ccc;
}
.code-highlight pre, .linenodiv pre {
    padding: 5px 2px 0px 5px;
    margin: 0;
}
.code-highlight pre div:target {
    background-color: #FFFFBE !important;
}
.linenos { padding: 0px !important; border:0px !important;}
.linenos a { text-decoration: none; }

.code { display: block; border:0px !important; }
.code-highlight .hll, .codehilite .hll { background-color: #ffffcc }
.code-highlight .c, .codehilite .c { color: #408080; font-style: italic } /* Comment */
.code-highlight .err, .codehilite .err { border: 1px solid #FF0000 } /* Error */
.code-highlight .k, .codehilite .k { color: #008000; font-weight: bold } /* Keyword */
.code-highlight .o, .codehilite .o { color: #666666 } /* Operator */
.code-highlight .cm, .codehilite .cm { color: #408080; font-style: italic } /* Comment.Multiline */
.code-highlight .cp, .codehilite .cp { color: #BC7A00 } /* Comment.Preproc */
.code-highlight .c1, .codehilite .c1 { color: #408080; font-style: italic } /* Comment.Single */
.code-highlight .cs, .codehilite .cs { color: #408080; font-style: italic } /* Comment.Special */
.code-highlight .gd, .codehilite .gd { color: #A00000 } /* Generic.Deleted */
.code-highlight .ge, .codehilite .ge { font-style: italic } /* Generic.Emph */
.code-highlight .gr, .codehilite .gr { color: #FF0000 } /* Generic.Error */
.code-highlight .gh, .codehilite .gh { color: #000080; font-weight: bold } /* Generic.Heading */
.code-highlight .gi, .codehilite .gi { color: #00A000 } /* Generic.Inserted */
.code-highlight .go, .codehilite .go { color: #808080 } /* Generic.Output */
.code-highlight .gp, .codehilite .gp { color: #000080; font-weight: bold } /* Generic.Prompt */
.code-highlight .gs, .codehilite .gs { font-weight: bold } /* Generic.Strong */
.code-highlight .gu, .codehilite .gu { color: #800080; font-weight: bold } /* Generic.Subheading */
.code-highlight .gt, .codehilite .gt { color: #0040D0 } /* Generic.Traceback */
.code-highlight .kc, .codehilite .kc { color: #008000; font-weight: bold } /* Keyword.Constant */
.code-highlight .kd, .codehilite .kd { color: #008000; font-weight: bold } /* Keyword.Declaration */
.code-highlight .kn, .codehilite .kn { color: #008000; font-weight: bold } /* Keyword.Namespace */
.code-highlight .kp, .codehilite .kp { color: #008000 } /* Keyword.Pseudo */
.code-highlight .kr, .codehilite .kr { color: #008000; font-weight: bold } /* Keyword.Reserved */
.code-highlight .kt, .codehilite .kt { color: #B00040 } /* Keyword.Type */
.code-highlight .m, .codehilite .m { color: #666666 } /* Literal.Number */
.code-highlight .s, .codehilite .s { color: #BA2121 } /* Literal.String */
.code-highlight .na, .codehilite .na { color: #7D9029 } /* Name.Attribute */
.code-highlight .nb, .codehilite .nb { color: #008000 } /* Name.Builtin */
.code-highlight .nc, .codehilite .nc { color: #0000FF; font-weight: bold } /* Name.Class */
.code-highlight .no, .codehilite .no { color: #880000 } /* Name.Constant */
.code-highlight .nd, .codehilite .nd { color: #AA22FF } /* Name.Decorator */
.code-highlight .ni, .codehilite .ni { color: #999999; font-weight: bold } /* Name.Entity */
.code-highlight .ne, .codehilite .ne { color: #D2413A; font-weight: bold } /* Name.Exception */
.code-highlight .nf, .codehilite .nf { color: #0000FF } /* Name.Function */
.code-highlight .nl, .codehilite .nl { color: #A0A000 } /* Name.Label */
.code-highlight .nn, .codehilite .nn { color: #0000FF; font-weight: bold } /* Name.Namespace */
.code-highlight .nt, .codehilite .nt { color: #008000; font-weight: bold } /* Name.Tag */
.code-highlight .nv, .codehilite .nv { color: #19177C } /* Name.Variable */
.code-highlight .ow, .codehilite .ow { color: #AA22FF; font-weight: bold } /* Operator.Word */
.code-highlight .w, .codehilite .w { color: #bbbbbb } /* Text.Whitespace */
.code-highlight .mf, .codehilite .mf { color: #666666 } /* Literal.Number.Float */
.code-highlight .mh, .codehilite .mh { color: #666666 } /* Literal.Number.Hex */
.code-highlight .mi, .codehilite .mi { color: #666666 } /* Literal.Number.Integer */
.code-highlight .mo, .codehilite .mo { color: #666666 } /* Literal.Number.Oct */
.code-highlight .sb, .codehilite .sb { color: #BA2121 } /* Literal.String.Backtick */
.code-highlight .sc, .codehilite .sc { color: #BA2121 } /* Literal.String.Char */
.code-highlight .sd, .codehilite .sd { color: #BA2121; font-style: italic } /* Literal.String.Doc */
.code-highlight .s2, .codehilite .s2 { color: #BA2121 } /* Literal.String.Double */
.code-highlight .se, .codehilite .se { color: #BB6622; font-weight: bold } /* Literal.String.Escape */
.code-highlight .sh, .codehilite .sh { color: #BA2121 } /* Literal.String.Heredoc */
.code-highlight .si, .codehilite .si { color: #BB6688; font-weight: bold } /* Literal.String.Interpol */
.code-highlight .sx, .codehilite .sx { color: #008000 } /* Literal.String.Other */
.code-highlight .sr, .codehilite .sr { color: #BB6688 } /* Literal.String.Regex */
.code-highlight .s1, .codehilite .s1 { color: #BA2121 } /* Literal.String.Single */
.code-highlight .ss, .codehilite .ss { color: #19177C } /* Literal.String.Symbol */
.code-highlight .bp, .codehilite .bp { color: #008000 } /* Name.Builtin.Pseudo */
.code-highlight .vc, .codehilite .vc { color: #19177C } /* Name.Variable.Class */
.code-highlight .vg, .codehilite .vg { color: #19177C } /* Name.Variable.Global */
.code-highlight .vi, .codehilite .vi { color: #19177C } /* Name.Variable.Instance */
.code-highlight .il, .codehilite .il { color: #666666 } /* Literal.Number.Integer.Long */
