## -*- coding: utf-8 -*-
<%inherit file="/base/base.html"/>

## don't trigger flash messages on this page
<%def name="flash_msg()">
</%def>

<%block name="title">
    ${_('%s Creating Repository') % c.repo_name}
</%block>

<%def name="breadcrumbs_links()">
    ${_('Creating repository')} ${c.repo}
</%def>

<%block name="header_menu">
    ${self.menu('repositories')}
</%block>
<%def name="main()">
<div class="box">
    <!-- box / title -->
    <div class="title">
        ${self.breadcrumbs()}
    </div>

    <div style="display:table; padding: 10px 0px; font-size: 14px;font-weight: bold;margin-right: auto;margin-left: auto">
        ${_('Repository "%(repo_name)s" is being created, you will be redirected when this process is finished.' % {'repo_name':c.repo_name})}
    </div>

    <div id="progress" style="width: 500px;margin-left: auto; margin-right: auto">
        <div class="progress progress-striped active">
          <div class="progress-bar progress-bar" role="progressbar"
               aria-valuenow="100" aria-valuemin="0" aria-valuemax="100" style="width: 100%">
          </div>
        </div>
    </div>
    <div id="progress_error" style="display: none;">
        <div style="font-weight: bold; color:#aa1111">
        ${_("We're sorry but error occurred during this operation. Please check your Kallithea server logs, or contact administrator.")}
        </div>
    </div>
</div>

<script>
(function worker() {
  $.ajax({
    url: '${h.url('repo_check_home', repo_name=c.repo_name, repo=c.repo, task_id=c.task_id)}',
    success: function(data) {
      if(data.result === true){
          //redirect to created fork if our ajax loop tells us to do so.
          window.location = "${h.url('summary_home', repo_name = c.repo)}";
      }
    },
    complete: function(resp, status) {
      if (resp.status == 200){
          // Schedule the next request when the current one's complete
          setTimeout(worker, 1000);
      }
      else{
          $("#progress").html($('#progress_error').html());
      }
    }
  });
})();
</script>
</%def>
