## -*- coding: utf-8 -*-
<%inherit file="/base/base.html"/>

<%block name="title">
    ${_('My Account')} ${c.authuser.username}
</%block>

<%def name="breadcrumbs_links()">
    ${_('My Account')}
</%def>

<%block name="header_menu">
    ${self.menu('admin')}
</%block>

<%def name="main()">
<div class="box" style="overflow:auto">
    <div class="title">
        ${self.breadcrumbs()}
    </div>

    ##main
    <div style="width: 150px; float:left">
        <ul class="nav nav-pills nav-stacked">
          <li>
           <div class="gravatar_box" style="height: 26px">
               <div class="gravatar" style="float: left">
                 ${h.gravatar(c.user.email)}
               </div>
               <div class="truncate" style="margin:10px 0px 10px 0px; color:#5f5f5f; float:left; width: 100px">
                <strong>${c.user.username}</strong>
               </div>
           </div>
          </li>
          <li class="${'active' if c.active=='profile' else ''}"><a href="${h.url('my_account')}">${_('Profile')}</a></li>
          <li class="${'active' if c.active=='emails' else ''}"><a href="${h.url('my_account_emails')}">${_('Email Addresses')}</a></li>
          <li class="${'active' if c.active=='password' else ''}"><a href="${h.url('my_account_password')}">${_('Password')}</a></li>
          <li class="${'active' if c.active=='api_keys' else ''}"><a href="${h.url('my_account_api_keys')}">${_('API Keys')}</a></li>
          <li class="${'active' if c.active=='repos' else ''}"><a href="${h.url('my_account_repos')}">${_('Owned Repositories')}</a></li>
          <li class="${'active' if c.active=='watched' else ''}"><a href="${h.url('my_account_watched')}">${_('Watched Repositories')}</a></li>
          <li class="${'active' if c.active=='perms' else ''}"><a href="${h.url('my_account_perms')}">${_('Show Permissions')}</a></li>
        </ul>
    </div>

    <div style="min-width:750px; float:left; padding: 10px 0px 0px 20px;margin: 0px 0px 0px 10px; border-left: 1px solid #DDDDDD">
        <%include file="/admin/my_account/my_account_${c.active}.html"/>
    </div>
</div>

</%def>
