<div class="file_author" style="clear:both">
    <div class="item">${h.literal(ungettext(u'%s author',u'%s authors',len(c.authors)) % ('<b>%s</b>' % len(c.authors))) }</div>
    %for email, user in c.authors:
      <div class="contributor tooltip" style="float:left" title="${user}">
        <div class="gravatar" style="margin:1px">
          ${h.gravatar(email, size=20)}
        </div>
      </div>
    %endfor
</div>
