##commit highligthing

%for cnt,sr in enumerate(c.formated_results):
    %if h.HasRepoPermissionAny('repository.write','repository.read','repository.admin')(sr['repository'],'search results check'):
    <div class="table">
        <div id="body${cnt}" class="codeblock">
            <div class="code-header">
                <div class="search-path">${h.link_to(h.literal('%s &raquo; %s' % (sr['repository'],sr['raw_id'])),
                h.url('changeset_home',repo_name=sr['repository'],revision=sr['raw_id']))}
                ${h.fmt_date(h.time_to_datetime(sr['date']))}
                </div>
            </div>
            <div class="left">
                <div class="author">
                    <div class="gravatar">
                      ${h.gravatar(h.email_or_none(sr['author']), size=20)}
                    </div>
                    <span>${h.person(sr['author'])}</span><br/>
                    <span>${h.email_or_none(sr['author'])}</a></span><br/>
                </div>
                %if sr['message_hl']:
                <div class="search-code-body">
                    <pre>${h.literal(sr['message_hl'])}</pre>
                </div>
                %else:
                <div class="message">${h.urlify_commit(sr['message'], sr['repository'])}</div>
                %endif
            </div>
        </div>
    </div>
    %else:
        %if cnt == 0:
        <div class="table">
            <div id="body${cnt}" class="codeblock">
                <div class="error">${_('Permission denied')}</div>
            </div>
        </div>
        %endif
    %endif
%endfor
%if c.cur_query and c.formated_results:
<div class="pagination-wh pagination-left">
    ${c.formated_results.pager('$link_previous ~2~ $link_next')}
</div>
%endif
