<%namespace name="dpb" file="/base/default_perms_box.html"/>
${dpb.default_perms_box(url('edit_user_perms', id=c.user.user_id))}

## permissions overview
<%namespace name="p" file="/base/perms_summary.html"/>
${p.perms_summary(c.perm_user.permissions, show_all=True)}
