${h.form(url('admin_settings_email'), method='post')}
    <div class="form">

        <div class="fields">
            <div class="field">
                <div class="label">
                    <label for="test_email">${_('Send test email to')}:</label>
                </div>
                <div class="input">
                    ${h.text('test_email',size=30)}
                </div>
            </div>

            <div class="buttons">
            ${h.submit('send',_('Send'),class_="btn")}
            </div>
        </div>
    </div>
${h.end_form()}
